close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get Display Resolution %%%%%%%%
Displ = groot();
ScW = Displ.ScreenSize(3);
ScH = Displ.ScreenSize(4);
FiW = 1024;
FiH = 720;
Cl0 = [0,102,204]/255;
Cl1 = [102,204,0]/255;
Cl2 = [204,0,102]/255;
Ms = 13; % MarkerSize
Lw = 2;  % LineWidth

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fibonacci LSFR Implementation
% Self-Synchronization without Feedback
% AURORA 8/10
% Gx = 1 + X3 + X4 + X5 + X16
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Some Data for Testing %%%%%%%
N = 128;
Var = nan(1,N);
Var(1:2:end) = 1;
Var(2:2:end) = 0;
Data = [zeros(1,2*N),ones(1,N)];
Data = [Data,Data,Var,Data,Data];
N = length(Data);

Scrambled   = 0;
Descrambled = 0;
SC    = zeros(1,16); 
DSC   = zeros(1,16); 
T_SC  = zeros(1,16);
T_DSC = zeros(1,16);

Scramled_Data    = zeros(1,N);
Descrambled_Data = zeros(1,N); 

for i = 1:N
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % LSFR - Based SCRAMBLING %%%%%%%
    T0 = xor(SC(5),SC(16));
    T1 = xor(SC(4),T0);
    T2 = xor(SC(3),T1);
    T3 = xor(Data(i),T2);
    
    T_SC(1)   = T3;
    T_SC(2)   = SC(1);
    T_SC(3)   = SC(2);
    T_SC(4)   = SC(3);
    T_SC(5)   = SC(4);
    T_SC(6)   = SC(5);
    T_SC(7)   = SC(6);
    T_SC(8)   = SC(7);
    T_SC(9)   = SC(8);
    T_SC(10)  = SC(9);
    T_SC(11)  = SC(10);
    T_SC(12)  = SC(11);
    T_SC(13)  = SC(12);
    T_SC(14)  = SC(13);
    T_SC(15)  = SC(14);
    T_SC(16)  = SC(15);
    Scrambled = T_SC(16);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % LSFR - Based DESCRAMBLING %%%%%%%
    T_DSC(1)    = Scrambled;
    T_DSC(2)    = DSC(1);
    T_DSC(3)    = DSC(2);
    T_DSC(4)    = DSC(3);
    T_DSC(5)    = DSC(4);
    T_DSC(6)    = DSC(5);
    T_DSC(7)    = DSC(6);
    T_DSC(8)    = DSC(7);
    T_DSC(9)    = DSC(8);
    T_DSC(10)   = DSC(9);
    T_DSC(11)   = DSC(10);
    T_DSC(12)   = DSC(11);
    T_DSC(13)   = DSC(12);
    T_DSC(14)   = DSC(13);
    T_DSC(15)   = DSC(14);
    T_DSC(16)   = DSC(15);
    
    T0 = xor(Scrambled,DSC(3));
    T1 = xor(T0,DSC(4));
    T2 = xor(T1,DSC(5));
    T3 = xor(T2,DSC(16));
    Descrambled   = T3;
    
    
    %%%%%%%%%%%%%%%%%%%%%
    % UPDATE CYCLE %%%%%%
    SC  = T_SC;
    DSC = T_DSC;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % SAVE FORVISUALIZATION %%%%%%%
    Scramled_Data(i)    = Scrambled;
    Descrambled_Data(i) = Descrambled;
    
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DISPLAY AND COMPARE %%%%%%%
figure('name','Aurora 8b/10b Xilinx Scrambling & Descrambling','position',[ScW/2-FiW/2,ScH/2-FiH/2,FiW,FiH]);
subplot(3,1,1);
plot(Data,'color',Cl0,'linestyle','-','markersize',Ms,'marker','.');
hold on;grid on;
ylim([-0.1,1.1]);
title('INPUT DATA');

subplot(3,1,2);
plot(Scramled_Data,'color',Cl1,'linestyle','-','markersize',Ms,'marker','.');
hold on;grid on;
ylim([-0.1,1.1]);
title('SCRAMBLED DATA');

subplot(3,1,3);
plot(Descrambled_Data,'color',Cl2,'linestyle','-','markersize',Ms,'marker','.');
hold on;grid on;
ylim([-0.1,1.1]);
title('DESCRAMBLED DATA');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute and Show Spectrum Differences %%%%%%%%
Randomness = 1e-6*randn(1,N);
SO = 20*log10(fftshift(abs(fft(Data + Randomness))));
SS = 20*log10(fftshift(abs(fft(Scramled_Data + Randomness))));

%%%%%%%%%%%%%%%%%%%
% Smooth the Data %
Order = 70;
b = 1/Order * ones(1,Order);
SS = filter(b,1,SS);
SO = filter(b,1,SO);
SS = SS(Order:end);
SO = SO(Order:end);
N = N - (Order - 1);

SO = SO - max(SO);
SS = SS - max(SS);
Freq = linspace(-1,1,N);
figure('name','Spectrum Comparison','position',[ScW/2-FiW/2,ScH/2-FiH/2,FiW,FiH]);
plot(Freq,SO,'color',Cl0,'linewidth',Lw);
hold on;grid on;
plot(Freq,SS,'color',Cl2,'linewidth',Lw);
ylim([-20,5]);
xlim([-0.5,0.5]);
ylabel('20log_{10}|S(\Omega)|');
xlabel('Normalized frequency x \pi rad/sample');
title('Scrambled / NonScrabled Signal Spectrum');
legend('Default','Scrambled');









